# Generated by Django 5.2.6 on 2025-09-17 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_registration",
            "0032_alter_historicalregisteredsubject_revision_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="additional_key",
            field=models.CharField(
                default="",
                editable=False,
                help_text="A uuid (or some other text value) to be added to bypass the unique constraint of just firstname, initials, and dob.The default constraint proves limiting since the source model usually has some other attribute in additional to first_name, initials and dob which is not captured in this model",
                max_length=36,
                verbose_name="-",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="comment",
            field=models.TextField(
                blank=True, default="", max_length=250, verbose_name="Comment"
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="dm_comment",
            field=models.CharField(
                default="",
                editable=False,
                max_length=150,
                verbose_name="Data Management comment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="ethnicity",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                default="",
                max_length=1,
                verbose_name="Gender",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="randomization_list_model",
            field=models.CharField(default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="registration_identifier",
            field=models.CharField(blank=True, default="", max_length=36),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="registration_status",
            field=models.CharField(
                blank=True,
                default="",
                max_length=25,
                verbose_name="Registration status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="relative_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="For example, mother's identifier, if available / appropriate",
                max_length=36,
                verbose_name="Identifier of immediate relation",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="screening_identifier",
            field=models.CharField(blank=True, default="", max_length=36),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="sid",
            field=models.CharField(
                blank=True, default="", max_length=15, verbose_name="SID"
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="subject_consent_id",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="subject_identifier_aka",
            field=models.CharField(
                default="",
                editable=False,
                help_text="track a previously allocated identifier.",
                max_length=50,
                verbose_name="Subject Identifier a.k.a",
            ),
        ),
        migrations.AlterField(
            model_name="historicalregisteredsubject",
            name="subject_type",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="additional_key",
            field=models.CharField(
                default="",
                editable=False,
                help_text="A uuid (or some other text value) to be added to bypass the unique constraint of just firstname, initials, and dob.The default constraint proves limiting since the source model usually has some other attribute in additional to first_name, initials and dob which is not captured in this model",
                max_length=36,
                verbose_name="-",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="comment",
            field=models.TextField(
                blank=True, default="", max_length=250, verbose_name="Comment"
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="dm_comment",
            field=models.CharField(
                default="",
                editable=False,
                max_length=150,
                verbose_name="Data Management comment",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="ethnicity",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                default="",
                max_length=1,
                verbose_name="Gender",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="randomization_list_model",
            field=models.CharField(default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="registration_identifier",
            field=models.CharField(blank=True, default="", max_length=36),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="registration_status",
            field=models.CharField(
                blank=True,
                default="",
                max_length=25,
                verbose_name="Registration status",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="relative_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="For example, mother's identifier, if available / appropriate",
                max_length=36,
                verbose_name="Identifier of immediate relation",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="screening_identifier",
            field=models.CharField(blank=True, default="", max_length=36),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="sid",
            field=models.CharField(
                blank=True, default="", max_length=15, verbose_name="SID"
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="subject_consent_id",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="subject_identifier_aka",
            field=models.CharField(
                default="",
                editable=False,
                help_text="track a previously allocated identifier.",
                max_length=50,
                verbose_name="Subject Identifier a.k.a",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="subject_type",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
    ]



from django.apps import apps as django_apps

from edc_label.label import Label


class PrintButtonAdminMixin:
    print_server_error = None

    def __init__(self, args, *kwargs):
        super(PrintButtonAdminMixin, self).__init__(*args, **kwargs)
        app_config = django_apps.get_app_config("edc_label")
        self._print_server = None
        self._printers = {}
        self.cups_server_ip = app_config.default_cups_server_ip
        self.label_templates = app_config.label_templates
        self.printer_label = app_config.default_printer_label

    def print_label(self, label_name, copies=None, context=None):
        copies = 1 if copies is None else copies
        label_template = self.label_templates.get(label_name)
        context = label_template.test_context if context is None else context
        label = Label(label_name)
        label.render_as_zpl_data(context=context, copies=copies)
        return label



from django.contrib import messages
from django.utils import timezone

from edc_dashboard.view_mixins import EdcViewMixin
from edc_lab import SHIPPED

from ...view_mixins import BoxViewMixin
from .action_view import ActionView


class VerifyBoxItemView(EdcViewMixin, BoxViewMixin, ActionView):
    post_action_url = "verify_box_listboard_url"
    box_item_failed = False
    valid_form_actions = ["verify_item", "reset_box", "verify_box"]

    @property
    def url_kwargs(self):
        return {
            "action_name": self.kwargs.get("action_name"),
            "box_identifier": self.box_identifier,
            "position": self.kwargs.get("position", "1"),
        }

    def process_form_action(self, request=None):
        if self.action == "verify_item":
            if self.box_item_identifier:
                self.verify_item()
        elif self.action == "reset_box":
            self.unverify_box()
        elif self.action == "verify_box":
            self.verify_box()

    def next_position(self):
        """Returns the next position relative to that from the URL."""
        self.kwargs["position"] = str(int(self.kwargs.get("position", "1")) + 1)

    def verify_item(self):
        """Updates the box_item as verified if the identifier matches
        the identifier already in that position.
        """
        if self.box.status == SHIPPED:
            message = "Unable to verify. Box has already been shipped."
            messages.error(self.request, message)
        else:
            box_item_in_position = self.get_box_item(position=self.kwargs.get("position"))
            self.redirect_querystring.update(alert=1)
            if box_item_in_position:
                if self.box_item:
                    if self.box_item == box_item_in_position:
                        box_item_in_position.verified = 1
                        box_item_in_position.verified_datetime = timezone.now()
                        self.next_position()
                        self.redirect_querystring.pop("alert")
                    else:
                        box_item_in_position.verified = -1
                        box_item_in_position.verified_datetime = None
                else:
                    box_item_in_position.verified = 0
                    box_item_in_position.verified_datetime = None
                box_item_in_position.save()
                self.box.save()

    def unverify_box(self):
        if self.box.status == SHIPPED:
            message = "Unable to reset. Box has already been shipped."
            messages.error(self.request, message)
        else:
            self.box.unverify_box()



from __future__ import annotations

from typing import TYPE_CHECKING

import pandas as pd
from django.apps import apps as django_apps
from django.db.models import Count
from django_pandas.io import read_frame

if TYPE_CHECKING:

    from ..models import Location


def get_stock_for_location_df(location: Location) -> pd.DataFrame:
    """Returns a dataframe of all stock records for this
    location.
    """
    stock_model_cls = django_apps.get_model("edc_pharmacy.Stock")
    qs_stock = (
        stock_model_cls.objects.values(
            "code",
            "unit_qty_in",
            "unit_qty_out",
            "confirmation",
            "allocation__registered_subject__subject_identifier",
            "stocktransferitem",
            "confirmationatsiteitem",
            "dispenseitem",
            "location__name",
        )
        .filter(location=location, qty=1)
        .annotate(count=Count("allocation__registered_subject__subject_identifier"))
    )
    df_stock = read_frame(qs_stock).rename(
        columns={
            "allocation__registered_subject__subject_identifier": "subject_identifier",
            "location__name": "location",
            # "confirmationatsiteitem": "confirmed_at_site",
            "count": "stock_qty",
        }
    )
    df_stock = df_stock.fillna(pd.NA)
    df_stock["unit_qty_bal"] = df_stock["unit_qty_in"] - df_stock["unit_qty_out"]
    df_stock["confirmed"] = False
    df_stock.loc[~df_stock["confirmation"].isna(), "confirmed"] = True
    df_stock["transferred"] = False
    df_stock.loc[~df_stock["stocktransferitem"].isna(), "transferred"] = True
    df_stock["confirmed_at_site"] = False
    df_stock.loc[~df_stock["confirmationatsiteitem"].isna(), "confirmed_at_site"] = True
    df_stock["dispensed"] = False
    df_stock.loc[~df_stock["dispenseitem"].isna(), "dispensed"] = True
    df_stock = df_stock.reset_index(drop=True)
    return df_stock



from django.apps import apps as django_apps
from edc_model_wrapper import ModelWrapper

from edc_lab.models import Manifest

edc_lab_app_config = django_apps.get_app_config("edc_lab")


class ManifestModelWrapper(ModelWrapper):
    model_cls = Manifest
    next_url_name = "manifest_listboard_url"



from textwrap import fill

import inflect
from django.contrib.auth import get_user_model
from reportlab.lib import colors
from reportlab.lib.units import cm
from reportlab.platypus.flowables import Spacer
from reportlab.platypus.para import Paragraph
from reportlab.platypus.tables import Table

from edc_constants.constants import OTHER
from edc_pdf_reports.crf_pdf_report import CrfPdfReport

from ..utils import get_adverse_event_app_label, get_ae_model

User = get_user_model()
p = inflect.engine()


class DeathPdfReport(CrfPdfReport):
    model = f"{get_adverse_event_app_label()}.deathreport"
    changelist_url = (
        f"{get_adverse_event_app_label()}_admin:{get_adverse_event_app_label()}_"
        "deathreport_changelist"
    )

    not_reported_text = "not reported"

    def get_report_story(self, **kwargs):
        story = []

        self.draw_demographics(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_section_one_header(story)

        self._draw_death_overview(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_opinion(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))
        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_audit_trail(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self.draw_end_of_report(story)

        return story

    def _draw_section_one_header(self, story):
        t = Table([["Section 1: Death Report"]], (18 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)
        t = Table([[f"Prepared by {self.get_user(self.model_obj)}."]], (18 * cm))
        self.set_table_style(t)
        story.append(t)

    def _draw_death_overview(self, story):
        # basics
        rows = [
            ["Reference:", self.model_obj.identifier],
            [
                "Report date:",
                self.model_obj.report_datetime.strftime("%Y-%m-%d %H:%M"),
            ],
            [
                "Death date:",
                getattr(self.model_obj, self.model_obj.death_date_field).strftime(
                    "%Y-%m-%d %H:%M"
                ),
            ],
            ["Study day:", self.model_obj.study_day or self.not_reported_text],
            [
                "Death as inpatient:",
                self.model_obj.death_as_inpatient or self.not_reported_text,
            ],
        ]

        t = Table(rows, (4 * cm, 14 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        t.hAlign = "LEFT"
        story.append(t)

    def _draw_opinion(self, story):
        t = Table([["Section 2: Opinion of Local Study Doctor"]], (18 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)
        rows = []

        row = ["Main cause of death:"]
        if not self.model_obj.cause_of_death:
            row.append(self.not_reported_text)
        elif self.cause_of_death == OTHER:
            row.append(
                fill(
                    f"{self.cause_of_death}: {self.cause_of_death_other}",
                    width=80,
                )
            )
        else:
            row.append(fill(self.cause_of_death))
        rows.append(row)

        t = Table(rows, (4 * cm, 14 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        t.hAlign = "LEFT"
        story.append(t)

        self.draw_narrative(story, title="Narrative:", text=self.model_obj.narrative)

    def _draw_audit_trail(self, story):
        s = self.styles["line_data_small"]
        t = Table(
            [
                [
                    Paragraph("Document", s),
                    Paragraph("Changed by", s),
                    Paragraph("Date", s),
                    Paragraph("Action", s),
                ]
            ],
            (3 * cm, 3 * cm, 3 * cm, 9 * cm),
        )
        self.set_table_style(t, bg_cmd=("BACKGROUND", (0, 0), (3, -1), colors.lightgrey))
        story.append(t)

        qs = (
            get_ae_model("deathreport")
            .history.filter(id=self.model_obj.id)
            .order_by("-history_date")
        )
        for obj in qs:
            username = obj.user_created if obj.history_type == "+" else obj.user_modified
            t = Table(
                [
                    [
                        Paragraph(get_ae_model("deathreport")._meta.verbose_name, s),
                        Paragraph(username, s),
                        Paragraph(obj.modified.strftime("%Y-%m-%d %H:%M"), s),
                        Paragraph(fill(self.history_change_message(obj), width=60), s),
                    ]
                ],
                (3 * cm, 3 * cm, 3 * cm, 9 * cm),
            )
            self.set_table_style(t)
            story.append(t)

    @property
    def cause_of_death(self):
        return getattr(self.model_obj.cause_of_death, "name", self.model_obj.cause_of_death)

    @property
    def cause_of_death_other(self):
        return getattr(self.model_obj, "cause_of_death_other", "")



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0089_alter_allocation_allocated_by_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allocation",
            name="allocation_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="confirmed_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="transfer_confirmation_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="transfer_confirmation_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="dispense_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="dispense_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="allocation_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="confirmed_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="transfer_confirmation_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="transfer_confirmation_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="dispense_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="dispense_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="receive_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="receive_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="repack_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="rx_date",
            field=models.DateField(
                default=django.utils.timezone.now, verbose_name="Date RX written"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="stock_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, help_text="date stock record created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="adjustment_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="request_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="request_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="transfer_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="transfer_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="bin_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="supplier_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="receive",
            name="receive_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="receive_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="repack_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="rx",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="rx",
            name="rx_date",
            field=models.DateField(
                default=django.utils.timezone.now, verbose_name="Date RX written"
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="stock_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, help_text="date stock record created"
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="adjustment_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stockavailability",
            name="created",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="request_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="request_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="transfer_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="transfer_item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="bin_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="item_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="supplier_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



from django.apps import apps as django_apps
from django.core.management.color import color_style

style = color_style()


def update_panels_on_post_migrate(sender, **kwargs):
    from edc_lab.site_labs import site_labs

    site_labs.migrated = True
    if site_labs.loaded:
        site_labs.update_panel_model(panel_model_cls=django_apps.get_model("edc_lab.panel"))



from __future__ import annotations

from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext


@admin.action(description="Scan items to transfer to site")
def transfer_stock_action(modeladmin, request, queryset):
    if queryset.count() > 1 or queryset.count() == 0:
        messages.add_message(
            request,
            messages.ERROR,
            gettext("Select one and only one item"),
        )
    else:
        url = reverse(
            "edc_pharmacy:transfer_stock_url",
            kwargs={"stock_transfer": queryset.first().pk},
        )
        return HttpResponseRedirect(url)
    return None



from __future__ import annotations

from typing import TYPE_CHECKING

from django import forms

from edc_sites.exceptions import InvalidSiteForSubjectError
from edc_sites.utils import valid_site_for_subject_or_raise

from ..exceptions import ActionError

if TYPE_CHECKING:
    from ..action import Action
    from ..models import ActionItem


class ActionItemModelFormMixin:
    def clean(self):
        cleaned_data = super().clean()
        self.valididate_site_for_subject_or_raise()
        action_cls = self._meta.model.get_action_cls()
        if self.action_cls:
            try:
                action_cls(
                    subject_identifier=self.get_subject_identifier(),
                    action_identifier=self.action_identifier,
                    related_action_item=self.related_action_item,
                    readonly=True,
                )
            except ActionError as e:
                raise forms.ValidationError(
                    f"{e.message!s}. Please contact your data manager."
                )
        return cleaned_data

    @property
    def action_cls(self) -> Action:
        return self._meta.model.get_action_cls()

    @property
    def action_identifier(self) -> str:
        return self.cleaned_data.get("action_identifier") or getattr(
            self.instance, "action_identifier", None
        )

    @property
    def related_action_item(self) -> ActionItem:
        return self.cleaned_data.get("related_action_item") or getattr(
            self.instance, "related_action_item", None
        )

    def valididate_site_for_subject_or_raise(self):
        if self.get_subject_identifier():
            try:
                valid_site_for_subject_or_raise(self.get_subject_identifier())
            except InvalidSiteForSubjectError:
                raise forms.ValidationError(
                    "Subject is not from this site. Please login to the correct site."
                )



from __future__ import annotations

from django.contrib import admin
from django.template.loader import render_to_string
from django_audit_fields.admin import audit_fieldset_tuple

from edc_action_item.fieldsets import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..forms import AeSusarForm
from ..templatetags.edc_adverse_event_extras import (
    format_ae_susar_description,
    select_description_template,
)
from .modeladmin_mixins import AdverseEventModelAdminMixin, NonAeInitialModelAdminMixin


class AeSusarModelAdminMixin(
    ModelAdminSubjectDashboardMixin,
    NonAeInitialModelAdminMixin,
    AdverseEventModelAdminMixin,
    ActionItemModelAdminMixin,
):
    form = AeSusarForm

    search_fields = (
        "subject_identifier",
        "action_identifier",
        "ae_initial__action_identifier",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "ae_initial",
                    "report_datetime",
                    "submitted_datetime",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {"report_status": admin.VERTICAL}

    def get_list_display(self, request) -> tuple[str]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "description",
            "initial_ae",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("report_datetime", "submitted_datetime")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    @staticmethod
    def description(obj=None) -> str:
        """Returns a formatted comprehensive description."""
        context = format_ae_susar_description({}, obj, 50)
        return render_to_string(select_description_template("aesusar"), context)



from __future__ import annotations

from django.db import models
from django.utils import timezone
from sequences import get_next_value

from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.model_mixins import SiteModelMixin

from .storage_bin import StorageBin


class StorageBinItem(SiteModelMixin, BaseUuidModel):
    item_identifier = models.CharField(
        max_length=36,
        unique=True,
        null=True,
        blank=True,
        help_text="A sequential unique identifier set by the EDC",
    )

    storage_bin = models.ForeignKey(StorageBin, on_delete=models.PROTECT)

    stock = models.OneToOneField(
        "edc_pharmacy.stock",
        on_delete=models.PROTECT,
        null=True,
        limit_choices_to={
            "confirmationatsiteitem__isnull": False,
            "dispenseitem__isnull": True,
        },
    )

    item_datetime = models.DateTimeField(default=timezone.now)

    history = HistoricalRecords()

    def __str__(self):
        return (
            f"{self.storage_bin.location.site.id}:"
            f"{self.storage_bin.bin_identifier}:{self.item_identifier}"
        )

    def save(self, *args, **kwargs):
        self.site = self.storage_bin.location.site
        if not self.id:
            self.item_identifier = f"{get_next_value(self._meta.label_lower):06d}"
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Storage bin item"
        verbose_name_plural = "Storage bin items"



from django.db import models
from django.urls import reverse
from django_crypto_fields.fields import EncryptedTextField

from edc_constants.constants import CLOSED, NEW
from edc_model.models import BaseUuidModel, HistoricalRecords

from ..constants import CANCELLED, EXPORTED
from ..model_mixins import ExportTrackingFieldsModelMixin


class ObjectHistoryManager(models.Manager):
    def get_by_natural_key(self, export_uuid):
        return self.get(export_uuid=export_uuid)


class ObjectHistory(ExportTrackingFieldsModelMixin, BaseUuidModel):
    model = models.CharField(max_length=64)

    tx_pk = models.UUIDField()

    tx = EncryptedTextField()

    exported_datetime = models.DateTimeField()

    timestamp = models.CharField(max_length=50, null=True, db_index=True)

    status = models.CharField(
        max_length=15,
        default=NEW,
        choices=(
            (NEW, "New"),
            (EXPORTED, "Exported"),
            (CLOSED, "Closed"),
            (CANCELLED, "Cancelled"),
        ),
        help_text="exported by export_transactions, closed by import_receipts",
    )

    received = models.BooleanField(default=False, help_text="True if ACK received")

    received_datetime = models.DateTimeField(null=True, help_text="date ACK received")

    is_ignored = models.BooleanField(default=False, help_text="Ignore if update")

    is_error = models.BooleanField(default=False)

    objects = ObjectHistoryManager()

    history = HistoricalRecords()

    def __str__(self):
        return f"{self.model_name} {self.status} {self.export_uuid}"

    def natural_key(self):
        return (self.export_uuid,)

    def render(self):
        url = reverse(
            "view_transaction_url",
            kwargs={
                "app_label": self._meta.app_label,
                "model_name": self._meta.model_name.lower(),
                "pk": self.pk,
            },
        )
        ret = (
            f'<a href="{url}" class="add-another" id="add_id_report" onclick="return '
            'showAddAnotherPopup(this);"> <img src="/static/admin/img/icon_addlink.gif" '
            'width="10" height="10" alt="View transaction"/></a>'
        )
        return ret

    render.allow_tags = True

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Object History"
        verbose_name_plural = "Object Histories"



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from edc_consent import site_consents

from .utils import get_subject_screening_model

if TYPE_CHECKING:
    from edc_consent.consent_definition import ConsentDefinition

    from .model_mixins import ScreeningModelMixin


class SubjectScreeningFormValidatorMixin:
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._subject_screening = None

    @property
    def screening_identifier(self):
        return self.cleaned_data.get("screening_identifier")

    @property
    def report_datetime(self):
        return self.cleaned_data.get("report_datetime")

    @property
    def subject_screening_model(self) -> str:
        return get_subject_screening_model()

    @property
    def subject_screening_model_cls(self) -> type[ScreeningModelMixin]:
        return django_apps.get_model(self.subject_screening_model)

    @property
    def subject_screening(self) -> ScreeningModelMixin:
        if not self._subject_screening:
            try:
                self._subject_screening = self.subject_screening_model_cls.objects.get(
                    screening_identifier=self.screening_identifier
                )
            except ObjectDoesNotExist:
                self.raise_validation_error(
                    'Complete the "Subject Screening" form before proceeding.',
                    error_code="missing_subject_screening",
                )
        return self._subject_screening

    def get_consent_definition_or_raise(self) -> ConsentDefinition:
        """Is there a single consent definition registered"""
        return site_consents.get_consent_definition(
            screening_model=self.instance._meta.label_lower,
            report_datetime=self.report_datetime,
        )



from __future__ import annotations

from typing import Any

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist


class PredicateError(Exception):
    pass


class NoValueError(Exception):
    pass


class BasePredicate:
    @staticmethod
    def get_value(attr: str = None, source_model: str | None = None, **kwargs) -> Any:
        """Returns a value by checking for the attr on each arg.

        Each arg in args may be a model instance, queryset, or None.

        If not found, does a lookup on the source_model.
        """
        found: bool = False
        value: Any = None
        for v in kwargs.values():
            try:
                value = getattr(v, attr)
            except AttributeError:
                continue
            else:
                found = True
                break
        if not found:
            visit = kwargs.get("visit")
            try:
                obj = django_apps.get_model(source_model).objects.get(
                    subject_visit__subject_identifier=visit.subject_identifier,
                    subject_visit__visit_schedule_name=visit.visit_schedule_name,
                    subject_visit__schedule_name=visit.schedule_name,
                    subject_visit__visit_code=visit.visit_code,
                    subject_visit__visit_code_sequence=visit.visit_code_sequence,
                    subject_visit__site=visit.site,
                )
            except ObjectDoesNotExist:
                value = None
            else:
                value = getattr(obj, attr)
        return value


class P(BasePredicate):
    """
    Simple predicate class.

    For example:

        predicate = P('gender', 'eq', 'MALE')
        predicate = P('referral_datetime', 'is not', None)
        predicate = P('age', '<=', 64)
    """

    funcs = {
        "is": lambda x, y: True if x is y else False,
        "is not": lambda x, y: True if x is not y else False,
        "gt": lambda x, y: True if x > y else False,
        ">": lambda x, y: True if x > y else False,
        "gte": lambda x, y: True if x >= y else False,
        ">=": lambda x, y: True if x >= y else False,
        "lt": lambda x, y: True if x < y else False,
        "<": lambda x, y: True if x < y else False,
        "lte": lambda x, y: True if x <= y else False,
        "<=": lambda x, y: True if x <= y else False,
        "eq": lambda x, y: True if x == y else False,
        "equals": lambda x, y: True if x == y else False,
        "==": lambda x, y: True if x == y else False,
        "neq": lambda x, y: True if x != y else False,
        "!=": lambda x, y: True if x != y else False,
        "in": lambda x, y: True if x in y else False,
    }

    def __init__(self, attr: str, operator: str, expected_value: list | str) -> None:
        self.attr = attr
        self.expected_value = expected_value
        self.func = self.funcs.get(operator)
        if not self.func:
            raise PredicateError(f"Invalid operator. Got {operator}.")
        self.operator = operator

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}({self.attr}, {self.operator}, "
            f"{self.expected_value})"
        )

    def __call__(self, **kwargs) -> bool:
        value = self.get_value(attr=self.attr, **kwargs)
        return self.func(value, self.expected_value)


class PF(BasePredicate):
    """
    Predicate with a lambda function.

    predicate = PF('age', lambda x: True if x >= 18 and x <= 64 else False)

    if lamda is anything more complicated just pass a func directly to the predicate attr:

        def my_func(visit, registered_subject, source_obj, source_qs):
            if source_obj.married and registered_subject.gender == FEMALE:
                return True
            return False

        class MyRuleGroup(RuleGroup):

            my_rule = Rule(
                ...
                predicate = my_func
                ...

    """

    def __init__(self, *attrs, func: callable = None) -> None:
        self.attrs = attrs
        self.func = func

    def __call__(self, **kwargs) -> Any:
        values = []
        for attr in self.attrs:
            values.append(self.get_value(attr=attr, **kwargs))
        return self.func(*values)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.attrs}, {self.func})"



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_audit_fields import audit_fieldset_tuple
from rangefilter.filters import DateRangeFilterBuilder

from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.list_filters import FutureDateListFilter
from edc_utils.date import to_local

from ...admin_site import edc_pharmacy_admin
from ...forms import StockRequestItemForm
from ...models import StockRequestItem
from ..actions.print_labels import print_labels_from_stock_request_item
from ..list_filters import (
    AssignmentListFilter,
    StockItemAllocationListFilter,
    StockItemTransferredListFilter,
    StockRequestItemPendingListFilter,
)
from ..model_admin_mixin import ModelAdminMixin
from ..remove_fields_for_blinded_users import remove_fields_for_blinded_users


class ApptDatetimeListFilter(FutureDateListFilter):
    title = "Appt date"

    parameter_name = "appt_datetime"
    field_name = "appt_datetime"


@admin.register(StockRequestItem, site=edc_pharmacy_admin)
class StockRequestItemAdmin(ModelAdminMixin, SimpleHistoryAdmin):
    change_list_title = "Pharmacy: Requested stock items"
    history_list_display = ()
    show_object_tools = False
    show_cancel = True
    list_per_page = 20
    form = StockRequestItemForm
    autocomplete_fields = ["rx"]
    actions = [print_labels_from_stock_request_item, "delete_selected"]

    fieldsets = (
        (
            "Section A",
            {
                "fields": (
                    "stock_request",
                    "request_item_datetime",
                    "rx",
                )
            },
        ),
        (
            "Section B",
            {
                "fields": ("allocation",),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "request_item_id",
        "item_date",
        "request_changelist",
        "allocated",
        "transferred",
        "location",
        "subject",
        "formulation",
        "allocation_changelist",
        "stock_changelist",
        "assignment",
    )

    list_filter = (
        ("request_item_datetime", DateRangeFilterBuilder()),
        "stock_request__location",
        StockItemAllocationListFilter,
        StockItemTransferredListFilter,
        AssignmentListFilter,
        StockRequestItemPendingListFilter,
        "visit_code",
        ApptDatetimeListFilter,
    )
    readonly_fields = ("rx", "allocation")

    search_fields = (
        "id",
        "registered_subject__subject_identifier",
        "stock_request__request_identifier",
        "allocation__id",
        "allocation__stock__code",
    )

    def get_list_display(self, request):
        fields = super().get_list_display(request)
        fields = remove_fields_for_blinded_users(request, fields)
        return fields

    def get_list_filter(self, request):
        fields = super().get_list_filter(request)
        fields = remove_fields_for_blinded_users(request, fields)
        return fields

    def get_search_fields(self, request):
        fields = super().get_search_fields(request)
        fields = remove_fields_for_blinded_users(request, fields)
        return fields

    @admin.display(description="Product")
    def formulation(self, obj):
        return format_html(
            "{}",
            mark_safe(
                f"{obj.stock_request.formulation}<BR>{obj.stock_request.container}"
            ),  # nosec B703, B308
        )

    @admin.display(description="Date", ordering="request_item_datetime")
    def item_date(self, obj):
        if obj.request_item_datetime:
            return to_local(obj.request_item_datetime).date()
        return None

    @admin.display(
        description="Allocation",
        ordering="allocation__registered_subject__subject_identifier",
    )
    def allocation_subject(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_allocation_changelist")
        url = f"{url}?q={obj.allocation.id}"
        context = dict(
            url=url,
            label=obj.allocation.registered_subject.subject_identifier,
            title="Go to allocation",
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Assignment", ordering="allocation__assignment")
    def assignment(self, obj):
        return obj.allocation.assignment

    @admin.display(description="A", boolean=True)
    def allocated(self, obj):
        if obj:
            return True if getattr(obj, "allocation", None) else False
        return None

    @admin.display(description="T", boolean=True)
    def transferred(self, obj):
        if obj and getattr(obj, "allocation", None):
            return True if obj.allocation.stock.stocktransferitem else False
        return False

    @admin.display(description="Subject", ordering="appt_datetime")
    def subject(self, obj):
        appt_date = to_local(obj.appt_datetime).date() if obj.appt_datetime else None
        context = dict(
            appt_date=appt_date,
            subject_identifier=obj.registered_subject.subject_identifier,
            visit_code_and_seq=f"{obj.visit_code}.{obj.visit_code_sequence}",
            changelist_url=reverse(
                "edc_pharmacy_admin:edc_pharmacy_stockrequestitem_changelist"
            ),
        )

        return render_to_string(
            "edc_pharmacy/stock/subject_list_display.html", context=context
        )

    @admin.display(description="Location", ordering="stock_request__location__name")
    def location(self, obj):
        return obj.stock_request.location

    @admin.display(description="ITEM#", ordering="request_item_identifier")
    def request_item_id(self, obj):
        return obj.request_item_identifier

    @admin.display(description="Request#")
    def request_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_stockrequest_changelist")
        url = f"{url}?q={obj.stock_request.request_identifier}"
        context = dict(
            url=url,
            label=f"{obj.stock_request.request_identifier}",
            title="Back to stock request",
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Allocation #")
    def allocation_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_allocation_changelist")
        url = f"{url}?q={obj.allocation.id}"
        context = dict(
            url=url,
            label=f"{obj.allocation.allocation_identifier}",
            title="Allocation",
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Stock #")
    def stock_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_stock_changelist")
        url = f"{url}?q={obj.allocation.stock.code}"
        context = dict(url=url, label=f"{obj.allocation.stock.code}", title="Go to stock")
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ("stock_request",)
        return self.readonly_fields

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "stock_request" and request.GET.get("stock_request"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("rx", 0)
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from edc_lab.constants import SHIPPED
from edc_lab.models import BoxItem

register = template.Library()


@register.inclusion_tag("edc_lab_dashboard/listboard/box/box_cell.html")
def show_box_rows(box, listboard_url, position=None):
    """Returns rendered HTML of a box as a dictionary of keys headers, rows.

    Usage::

        {% block results_body %}
            {% show_box_rows box listboard_url position=position %}
        {% endblock results_body %}

    """
    position = "0" if position is None else str(position)
    btn_style = {-1: "btn-danger", 0: "btn-default", 1: "btn-success"}
    pos = 0
    rows = []
    header = range(1, box.box_type.across + 1)
    for i in range(1, box.box_type.down + 1):
        row = {"position": i, "cells": []}
        for _ in range(1, box.box_type.across + 1):
            cell = {}
            pos += 1
            try:
                box_item = box.boxitem_set.get(position=pos)
            except ObjectDoesNotExist:
                box_item = BoxItem(box=box)
            cell["btn_style"] = btn_style.get(box_item.verified)
            cell["btn_label"] = str(pos).zfill(2)
            cell["btn_title"] = box_item.human_readable_identifier or "empty"
            cell["has_focus"] = str(pos) == position
            cell["box_item"] = box_item
            row["cells"].append(cell)
        rows.append(row)
    return {"headers": header, "rows": rows}


@register.filter(is_safe=True)
def verified(box_item):
    """Returns a safe HTML check mark string if a Box item has been verified."""
    is_verified = False
    if box_item.verified:
        if int(box_item.verified) == 1:
            is_verified = True
        elif int(box_item.verified) == -1:
            is_verified = False
    return (
        ""
        if not is_verified
        else mark_safe(
            render_to_string("edc_lab_dashboard/controls/verified_check.html")
        )  # nosec B703 B308
    )


@register.filter(is_safe=True)
def shipped(box_item):
    """Returns a safe HTML check mark string if a Box item has been shipped."""
    return (
        ""
        if not box_item.status == SHIPPED
        else mark_safe(
            render_to_string("edc_lab_dashboard/controls/verified_shipped.html")
        )  # nosec B703 B308
    )


@register.inclusion_tag("edc_lab_dashboard/listboard/tags/status_column.html")
def status_column(model_wrapper, *attrs):
    options = {}
    for attr in attrs:
        try:
            options.update({attr: True if getattr(model_wrapper, attr) else False})
        except AttributeError:
            pass
    return dict(options=options)



# from .action import Action
# from .action_with_notification import ActionWithNotification
# from .create_action_item import SingletonActionItemError, create_action_item
# from .decorators import register
# from .delete_action_item import ActionItemDeleteError, delete_action_item
# from .fieldsets import action_fields, action_fieldset_tuple
# from .modeladmin_mixins import ActionItemModelAdminMixin
# from .site_action_items import site_action_items




from django.contrib import admin

from edc_dx import get_diagnosis_labels_prefixes


def get_clinical_review_cond_radio_fields() -> dict[str, int]:
    radio_fields = {}
    for prefix in get_diagnosis_labels_prefixes():
        cond = prefix.lower()
        radio_fields.update(
            {
                f"{cond}_dx": admin.VERTICAL,
                f"{cond}_test": admin.VERTICAL,
            }
        )
    return radio_fields


def get_clinical_review_baseline_cond_radio_fields() -> dict[str, int]:
    radio_fields = {}
    for prefix in get_diagnosis_labels_prefixes():
        cond = prefix.lower()
        radio_fields.update(
            {
                f"{cond}_dx": admin.VERTICAL,
                f"{cond}_dx_at_screening": admin.VERTICAL,
            }
        )
    return radio_fields



from edc_model.models import BaseUuidModel

from ..model_mixins import RandomizationListModelMixin


class RandomizationList(RandomizationListModelMixin, BaseUuidModel):
    class Meta(RandomizationListModelMixin.Meta, BaseUuidModel.Meta):
        pass



import sys

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.checks import Warning
from django.db.models import Q
from django.db.utils import OperationalError, ProgrammingError


def edc_notification_check(app_configs, **kwargs):
    errors = []
    if not getattr(settings, "EMAIL_ENABLED", False):
        errors.append(
            Warning(
                "Notifications by email are disabled.",
                hint="To enable set settings.EMAIL_ENABLED = True",
                id="edc_notification.W002",
            )
        )
    try:
        if "migrate" not in sys.argv and "makemigrations" not in sys.argv:
            users = get_user_model().objects.filter(
                (
                    Q(first_name__isnull=True)
                    | Q(last_name__isnull=True)
                    | Q(email__isnull=True)
                ),
                is_active=True,
                is_staff=True,
            )
            try:
                for user in users:
                    errors.append(
                        Warning(
                            (
                                f"User account is incomplete. Check that first name, "
                                f"last name and email are complete. See {user}"
                            ),
                            hint="Complete the user's account details.",
                            obj=get_user_model(),
                            id="edc_notification.W001",
                        )
                    )
            except OperationalError:
                pass
    except ProgrammingError:
        pass
    return errors



from ..listboard_filters import AliquotListboardViewFilters
from .base_listboard_view import BaseListboardView


class AliquotListboardView(BaseListboardView):
    form_action_url = "aliquot_form_action_url"
    listboard_template = "aliquot_listboard_template"
    listboard_url = "aliquot_listboard_url"
    listboard_view_filters = AliquotListboardViewFilters()
    listboard_view_permission_codename = "edc_lab_dashboard.view_lab_aliquot_listboard"
    listboard_view_only_my_permission_codename = None
    listboard_model = "edc_lab.aliquot"
    navbar_selected_item = "aliquot"
    search_form_url = "aliquot_listboard_url"
    show_all = True



from reportlab.lib import colors
from reportlab.platypus import Flowable


class TextFieldFlowable(Flowable):
    def __init__(
        self,
        name: str,
        width: int | None = None,
        height: int | None = None,
        value: str = "",
        **kwargs,
    ):
        Flowable.__init__(self)
        self.name = name
        self.width = width
        self.height = height
        self.value = value
        self.extra_options = kwargs

    def draw(self):
        # Calculate the position based on the current coordinates
        x = self.canv._currentMatrix[4]
        y = self.canv._currentMatrix[5]

        options = dict(
            name=self.name,
            tooltip=self.name,
            x=x,
            y=y,
            width=self.width,
            height=self.height,
            borderStyle="underlined",
            fillColor=colors.white,
            textColor=colors.black,
            forceBorder=False,
            value=self.value,
        )
        options.update(self.extra_options)
        self.canv.acroForm.textfield(**options)

    def split(self, availWidth, availHeight):
        return [self]



from django.contrib.sites.managers import CurrentSiteManager as BaseCurrentSiteManager


class CurrentSiteManager(BaseCurrentSiteManager):
    use_in_migrations = True

    def get_by_natural_key(self, subject_identifier):
        return self.get(subject_identifier=subject_identifier)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.html import format_html
from django.views.generic.base import TemplateView

from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin
from edc_protocol.view_mixins import EdcProtocolViewMixin

from ..models import Stock
from ..utils import confirm_stock

if TYPE_CHECKING:
    from ..models import Receive, RepackRequest


@method_decorator(login_required, name="dispatch")
class ConfirmStockFromInstanceView(
    EdcViewMixin, NavbarViewMixin, EdcProtocolViewMixin, TemplateView
):
    template_name: str = "edc_pharmacy/stock/confirm_stock_by_instance.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "pharmacy"

    def get_context_data(self, **kwargs):
        dct = self.get_values_dict(**kwargs)
        confirmed_count = (
            Stock.objects.values("code")
            .filter(**{dct.get("fk_attr"): dct.get("obj").id, "confirmed": True})
            .count()
        )

        unconfirmed_count = (
            Stock.objects.values("pk")
            .filter(
                **{dct.get("fk_attr"): dct.get("obj").id},
                confirmation__isnull=True,
            )
            .count()
        )
        unconfirmed_count = min(unconfirmed_count, 12)

        kwargs.update(
            source_identifier=dct.get("source_identifier"),
            source_model_name=self.model_cls._meta.verbose_name,
            source_changelist_url=self.source_changelist_url,
            source_pk=self.kwargs.get("source_pk"),
            item_count=list(range(1, unconfirmed_count + 1)),
            unconfirmed_count=unconfirmed_count,
            confirmed_count=confirmed_count,
            confirmed_codes=self.get_confirmed_codes(dct.get("obj"), dct.get("fk_attr")),
        )
        return super().get_context_data(**kwargs)

    def get_values_dict(self, **kwargs) -> dict:
        values_dict = {}
        obj = self.model_cls.objects.get(pk=kwargs.get("source_pk"))
        values_dict.update(obj=obj)
        try:
            values_dict.update(source_identifier=obj.repack_identifier)
            values_dict.update(fk_attr="repack_request")
        except AttributeError:
            values_dict.update(source_identifier=obj.receive_identifier)
            values_dict.update(fk_attr="receive_item__receive")
        return values_dict

    def get_confirmed_codes(self, obj: RepackRequest | Receive, fk_attr: str) -> list[str]:
        return (
            Stock.objects.values_list("code", flat=True)
            .filter(**{fk_attr: obj.id, "confirmed": True})
            .order_by("confirmed_datetime")
        )

    @property
    def source_changelist_url(self):
        return reverse(
            f"edc_pharmacy_admin:edc_pharmacy_{self.kwargs.get('model')}_changelist"
        )

    @property
    def model_cls(self):
        return django_apps.get_model(f"edc_pharmacy.{self.kwargs.get('model')}")

    def post(self, request, *args, **kwargs):
        dct = self.get_values_dict(**kwargs)
        codes = request.POST.getlist("codes")
        confirmed, already_confirmed, invalid = confirm_stock(
            dct.get("obj"),
            codes,
            dct.get("fk_attr"),
            confirmed_by=request.user.username,
            user_created=request.user.username,
        )
        msg = [
            f"Confirmed {len(confirmed)} stock records. ",
            f"Skipped {len(already_confirmed)} items already confirmed.",
        ]
        if len(invalid) > 0:
            msg.append(f"Skipped {len(invalid)} invalid codes.")
        messages.add_message(request, messages.SUCCESS, format_html("<BR>".join(msg)))
        if (
            Stock.objects.values("code")
            .filter(**{dct.get("fk_attr"): dct.get("obj").id, "confirmed": False})
            .exists()
        ):
            url = reverse("edc_pharmacy:confirm_stock_from_instance_url", kwargs=kwargs)
        else:
            url = f"{self.source_changelist_url}?q={dct.get('obj').pk}"
        return HttpResponseRedirect(url)



from django.db import models
from django.utils import timezone

from edc_constants.choices import YES_NO, YES_NO_UNKNOWN
from edc_constants.constants import NOT_APPLICABLE
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model import models as edc_models
from edc_model.validators import date_not_future, datetime_not_future
from edc_model_fields.fields import IsDateEstimatedField
from edc_protocol.validators import (
    date_not_before_study_start,
    datetime_not_before_study_start,
)


class HospitalizationModelMixin(NonUniqueSubjectIdentifierFieldMixin, models.Model):
    report_datetime = models.DateTimeField(
        verbose_name="Report Date",
        validators=[datetime_not_before_study_start, datetime_not_future],
        default=timezone.now,
    )

    have_details = models.CharField(
        verbose_name="Do you have details of the hospitalization?",
        max_length=15,
        choices=YES_NO,
    )

    admitted_date = models.DateField(
        verbose_name="When was the patient admitted?",
        validators=[date_not_future, date_not_before_study_start],
    )

    admitted_date_estimated = IsDateEstimatedField(
        verbose_name="Is this date estimated?",
    )

    discharged = models.CharField(
        verbose_name="Has the patient been discharged?",
        max_length=15,
        choices=YES_NO_UNKNOWN,
    )

    discharged_date = models.DateField(
        verbose_name="If YES, give date discharged",
        validators=[date_not_future, date_not_before_study_start],
        null=True,
        blank=True,
    )

    discharged_date_estimated = edc_models.IsDateEstimatedFieldNa(
        verbose_name="If YES, is this date estimated?",
        default=NOT_APPLICABLE,
    )

    narrative = models.TextField(
        verbose_name="Narrative", max_length=500, blank=True, default=""
    )

    class Meta(NonUniqueSubjectIdentifierFieldMixin.Meta):
        abstract = True
        verbose_name = "Hospitalization"
        verbose_name_plural = "Hospitalization"
        indexes = NonUniqueSubjectIdentifierFieldMixin.Meta.indexes



from .medication_order_model_mixin import MedicationOrderModelMixin
from .previous_model_mixin import PreviousNextModelMixin
from .study_medication_crf_model_mixin import StudyMedicationCrfModelMixin
from .study_medication_refill_model_mixin import StudyMedicationRefillModelMixin



from zoneinfo import ZoneInfo

from dateutil.relativedelta import relativedelta
from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ValidationError

from edc_model.validators import datetime_not_future
from edc_protocol.validators import datetime_not_before_study_start
from edc_utils.text import convert_php_dateformat


class CrfReportDateAllowanceError(Exception):
    pass


class CrfReportDateBeforeStudyStart(Exception):  # noqa: N818
    pass


class CrfReportDateIsFuture(Exception):  # noqa: N818
    pass


class CrfDateValidator:
    report_datetime_allowance = None
    allow_report_datetime_before_visit = False

    def __init__(
        self,
        report_datetime=None,
        visit_report_datetime=None,
        report_datetime_allowance=None,
        allow_report_datetime_before_visit=None,
        created=None,
        modified=None,
        subject_identifier=None,
    ):
        self.allow_report_datetime_before_visit = (
            allow_report_datetime_before_visit or self.allow_report_datetime_before_visit
        )
        self.report_datetime_allowance = (
            report_datetime_allowance or self.report_datetime_allowance
        )
        if not self.report_datetime_allowance:
            app_config = django_apps.get_app_config("edc_visit_tracking")
            self.report_datetime_allowance = app_config.report_datetime_allowance
        self.report_datetime = report_datetime.astimezone(ZoneInfo("UTC"))
        self.visit_report_datetime = visit_report_datetime.astimezone(ZoneInfo("UTC"))
        self.created = created
        self.modified = modified
        self.subject_identifier = subject_identifier
        self.validate()

    def validate(self):
        # datetime_not_before_study_start
        try:
            datetime_not_before_study_start(self.report_datetime)
        except ValidationError as e:
            message = e.message if hasattr(e, "message") else str(e)
            raise CrfReportDateBeforeStudyStart(message) from e
        # datetime_not_future
        try:
            datetime_not_future(self.report_datetime)
        except ValidationError as e:
            message = e.message if hasattr(e, "message") else str(e)
            raise CrfReportDateIsFuture(message) from e

        formatted_visit_datetime = self.visit_report_datetime.strftime(
            convert_php_dateformat(settings.SHORT_DATE_FORMAT)
        )

        # not before the visit report_datetime
        if (
            not self.allow_report_datetime_before_visit
            and self.report_datetime.date() < self.visit_report_datetime.date()
        ):
            raise CrfReportDateAllowanceError(
                "Report datetime may not be before the visit report datetime. "
                f"Visit report datetime is {formatted_visit_datetime}. "
            )

        # not more than x days greater than the visit report_datetime
        # if self.report_datetime_allowance > 0:
        max_allowed_report_datetime = self.visit_report_datetime + relativedelta(
            days=self.report_datetime_allowance
        )
        if self.report_datetime.date() > max_allowed_report_datetime.date():
            diff = (
                max_allowed_report_datetime.date() - self.visit_report_datetime.date()
            ).days
            raise CrfReportDateAllowanceError(
                "Report datetime may not be more than "
                f"{self.report_datetime_allowance} "
                f"days greater than the visit report datetime. Got {diff} days."
                f"Visit report datetime is {formatted_visit_datetime}. "
                f"See also AppConfig.report_datetime_allowance."
            )



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple

from ...admin_site import edc_pharmacy_admin
from ...forms import FormulationForm
from ...models import Formulation
from ..model_admin_mixin import ModelAdminMixin


@admin.register(Formulation, site=edc_pharmacy_admin)
class FormulationAdmin(ModelAdminMixin, admin.ModelAdmin):
    show_object_tools = True

    autocomplete_fields = ["medication"]

    form = FormulationForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "medication",
                    "strength",
                    "units",
                    "formulation_type",
                    "route",
                    "description",
                )
            },
        ),
        (
            "Investigational medicinal product",
            {
                "fields": (
                    "imp",
                    "imp_description",
                )
            },
        ),
        (
            "Notes",
            {"fields": ("notes",)},
        ),
        audit_fieldset_tuple,
    )

    radio_fields = {
        "units": admin.VERTICAL,
        "formulation_type": admin.VERTICAL,
        "route": admin.VERTICAL,
    }

    list_filter: tuple[str, ...] = (
        "imp",
        "strength",
        "units",
        "formulation_type",
        "route",
    )

    list_display = (
        "description",
        "medication",
        "strength",
        "units",
        "formulation_type",
        "route",
    )

    search_fields: tuple[str, ...] = ("medication__name",)

    ordering: tuple[str, ...] = ("medication__name",)



from textwrap import fill

import inflect
from reportlab.lib import colors
from reportlab.lib.units import cm
from reportlab.platypus import Paragraph, Table
from reportlab.platypus.flowables import KeepTogether, Spacer

from edc_constants.constants import OTHER, YES
from edc_pdf_reports.crf_pdf_report import CrfPdfReport

from ..utils import get_adverse_event_app_label, get_ae_model

p = inflect.engine()


class AePdfReport(CrfPdfReport):
    model = f"{get_adverse_event_app_label()}.aeinitial"
    changelist_url = (
        f"{get_adverse_event_app_label()}_admin:{get_adverse_event_app_label()}_"
        "aeinitial_changelist"
    )

    def get_report_story(self, **kwargs):
        story = []

        self.draw_demographics(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))
        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_section_one_header(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_ae_overview(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_ae_descripion(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_ae_drug_relationship(story)

        self._draw_ae_cause(story)

        self._draw_ae_action_taken(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))
        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_section_two(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self._draw_audit_trail(story)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        self.draw_end_of_report(story)

        return story

    def _draw_section_one_header(self, story):
        t = Table([["Section 1: Initial AE Report"]], (18 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)
        t = Table([[f"Prepared by {self.get_user(self.model_obj)}."]], (18 * cm))
        self.set_table_style(t)
        story.append(t)

    def _draw_ae_overview(self, story):
        # basics
        classification_text = fill(self.model_obj.ae_classification.name, width=80)
        if self.model_obj.ae_classification.name == OTHER:
            classification_text = fill(
                f"{classification_text}: {self.model_obj.ae_classification_other}",
                width=80,
            )

        sae_reason = (
            fill(f": {self.model_obj.sae_reason.name}", width=80)
            if self.model_obj.sae == YES
            else ""
        )
        sae_text = f"{self.model_obj.get_sae_display()}{sae_reason}"

        susar_reported = ""
        if self.model_obj.susar == YES:
            susar_reported = (
                ": reported" if self.model_obj.susar_reported == YES else ": not reported"
            )
        susar_text = f"{self.model_obj.get_susar_display()}{susar_reported}"

        rows = [
            ["Reference:", self.model_obj.identifier],
            [
                "Report date:",
                self.model_obj.report_datetime.strftime("%Y-%m-%d %H:%M"),
            ],
            ["Awareness date:", self.model_obj.ae_awareness_date.strftime("%Y-%m-%d")],
            ["Actual start date:", self.model_obj.ae_start_date.strftime("%Y-%m-%d")],
            ["Classification:", classification_text],
            ["Severity:", self.model_obj.get_ae_grade_display()],
            ["SAE:", sae_text],
            ["SUSAR:", susar_text],
        ]

        t = Table(rows, (4 * cm, 14 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        t.hAlign = "LEFT"
        story.append(t)

    def _draw_ae_descripion(self, story):
        self.draw_narrative(
            story, title="Description of AE:", text=self.model_obj.ae_description
        )

    def _draw_ae_drug_relationship(self, story):
        # relationship
        rows = [
            [
                "Is the incident related to the patient involvement in the study?",
                self.model_obj.get_ae_study_relation_possibility_display(),
            ],
            [
                "Relationship to study drug:",
                self.model_obj.get_study_drug_relation_display(),
            ],
        ]
        t = Table(rows, (14 * cm, 4 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)

    def _draw_ae_cause(self, story):
        # cause (Part3)
        left_width = 40 if self.model_obj.ae_cause == YES else 80
        rows = [
            fill(
                "Has a reason other than the specified study drug been "
                "identified as the cause of the event(s)?",
                width=left_width,
            )
        ]
        if self.model_obj.ae_cause == YES:
            rows.append(
                fill(
                    f"{self.model_obj.get_ae_cause_display()}: "
                    f"{self.model_obj.ae_cause_other}",
                    width=65,
                )
            )
            table_dimensions = (7 * cm, 11 * cm)
        else:
            rows.append(self.model_obj.get_ae_cause_display())
            table_dimensions = (14 * cm, 4 * cm)
        t = Table([rows], table_dimensions)
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

    def _draw_ae_action_taken(self, story):
        self.draw_narrative(
            story,
            title="Action taken for treatment of AE:",
            text=self.model_obj.ae_treatment,
        )

        story.append(Spacer(0.1 * cm, 0.5 * cm))

    def _draw_section_two(self, story):
        t1 = Table([["Section 2: Follow-up Reports"]], (18 * cm))
        self.set_table_style(t1, bg_cmd=self.bg_cmd)
        total = self.model_obj.ae_follow_ups.count()
        row_text = f"There {p.plural_verb('is', total)} {p.no('follow-up report', total)}."
        t2 = Table([[row_text]], (18 * cm))
        self.set_table_style(t2)
        story.append(KeepTogether([t1, t2]))

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        for index, obj in enumerate(self.model_obj.ae_follow_ups):
            self._draw_followup_story(story, obj, index + 1, total)
            story.append(Spacer(0.1 * cm, 0.5 * cm))
            story.append(Spacer(0.1 * cm, 0.5 * cm))

    def _draw_audit_trail(self, story):
        s = self.styles["line_data_small"]
        t = Table(
            [
                [
                    Paragraph("Document", s),
                    Paragraph("Changed by", s),
                    Paragraph("Date", s),
                    Paragraph("Action", s),
                ]
            ],
            (3 * cm, 3 * cm, 3 * cm, 9 * cm),
        )
        self.set_table_style(t, bg_cmd=("BACKGROUND", (0, 0), (3, -1), colors.lightgrey))
        story.append(t)

        followups = self.model_obj.ae_follow_ups.order_by("-created")
        index = followups.count()
        for followup in followups:
            qs = followup.history.filter(id=followup.id).order_by("-history_date")
            for obj in qs:
                username = obj.user_created if obj.history_type == "+" else obj.user_modified
                t = Table(
                    [
                        [
                            Paragraph(
                                f"{get_ae_model('aefollowup')._meta.verbose_name}: {index}",
                                s,
                            ),
                            Paragraph(username, s),
                            Paragraph(obj.modified.strftime("%Y-%m-%d %H:%M"), s),
                            Paragraph(fill(self.history_change_message(obj), width=60), s),
                        ]
                    ],
                    (3 * cm, 3 * cm, 3 * cm, 9 * cm),
                )
                self.set_table_style(t)
                story.append(t)
            index -= 1
        qs = (
            get_ae_model("aeinitial")
            .history.filter(id=self.model_obj.id)
            .order_by("-history_date")
        )
        for obj in qs:
            username = obj.user_created if obj.history_type == "+" else obj.user_modified
            t = Table(
                [
                    [
                        Paragraph(get_ae_model("aeinitial")._meta.verbose_name, s),
                        Paragraph(username, s),
                        Paragraph(obj.modified.strftime("%Y-%m-%d %H:%M"), s),
                        Paragraph(fill(self.history_change_message(obj), width=60), s),
                    ]
                ],
                (3 * cm, 3 * cm, 3 * cm, 9 * cm),
            )
            self.set_table_style(t)
            story.append(t)

    def _draw_followup_story(self, story, obj, index, total):
        rows = [
            ["Reference:", obj.identifier],
            ["Report date:", obj.report_datetime.strftime("%Y-%m-%d %H:%M")],
            ["Outcome date:", obj.outcome_date.strftime("%Y-%m-%d")],
            ["Outcome:", obj.get_outcome_display()],
            ["Severity increase:", obj.get_ae_grade_display()],
            ["Follow-up pending:", obj.get_followup_display()],
        ]

        t = Table([[f"Follow-up Report {index}/{total}"]], (18 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)
        t = Table([[f"Prepared by {self.get_user(obj)}."]], (18 * cm))
        self.set_table_style(t)
        story.append(t)
        t = Table(rows, (4 * cm, 14 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        story.append(t)

        self.draw_narrative(
            story,
            title="Description of follow-up AE outcome:",
            text=obj.relevant_history,
        )

    @property
    def title(self):
        return f"ADVERSE EVENT REPORT FOR {self.model_obj.subject_identifier}"



from django.db import models

from edc_constants.constants import CLOSED, OPEN


class AeSusarMethodsModelMixin(models.Model):
    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.action_identifier[-9:]}"

    def save(self, *args, **kwargs):
        self.subject_identifier = self.ae_initial.subject_identifier
        if self.submitted_datetime:
            self.report_status = CLOSED
            self.report_closed_datetime = self.submitted_datetime
        else:
            self.report_status = OPEN
            self.report_closed_datetime = None
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.action_identifier,)



# Generated by Django 2.1 on 2018-08-13 16:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_registration", "0012_auto_20180116_1528")]

    operations = [
        migrations.AlterUniqueTogether(
            name="registeredsubject",
            unique_together={("first_name", "dob", "initials", "additional_key")},
        ),
        migrations.AddIndex(
            model_name="registeredsubject",
            index=models.Index(
                fields=["first_name", "dob", "initials", "additional_key"],
                name="edc_registr_first_n_a70796_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="registeredsubject",
            index=models.Index(
                fields=["identity", "subject_identifier", "screening_identifier"],
                name="edc_registr_identit_eb36e0_idx",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-11-27 16:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_pharmacy",
            "0053_alter_location_managers_alter_historicalstock_lot_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalreceiveitem",
            name="task_id",
            field=models.UUIDField(null=True),
        ),
        migrations.AddField(
            model_name="receiveitem",
            name="task_id",
            field=models.UUIDField(null=True),
        ),
    ]



from __future__ import annotations

from datetime import UTC
from typing import TYPE_CHECKING

import pandas as pd
from celery import shared_task
from django.apps import apps as django_apps
from django.utils import timezone
from sequences import get_next_value

if TYPE_CHECKING:
    from uuid import UUID


@shared_task
def bulk_create_stock_request_items(
    stock_request_pk: UUID,
    nostock_as_dict: dict,
    user_created: str | None = None,
    bulk_create: bool | None = None,
) -> None:
    bulk_create = True if bulk_create is None else bulk_create
    stock_request_model_cls = django_apps.get_model("edc_pharmacy.StockRequest")
    stock_request_item_model_cls = django_apps.get_model("edc_pharmacy.StockRequestItem")
    registered_subject_model_cls = django_apps.get_model("edc_registration.registeredsubject")
    rx_model_cls = django_apps.get_model("edc_pharmacy.rx")

    stock_request = stock_request_model_cls.objects.get(pk=stock_request_pk)
    df_nostock = pd.DataFrame(nostock_as_dict)
    now = timezone.now()
    data = []
    for i, row in df_nostock[df_nostock.stock_qty == 0].iterrows():
        registered_subject = registered_subject_model_cls.objects.get(
            id=row["registered_subject_id"]
        )
        rx = rx_model_cls.objects.get(registered_subject=registered_subject)
        visit_code = str(int(row["next_visit_code"]))
        visit_code_sequence = int(10 * row["next_visit_code"] % 1)
        appt_datetime = row["next_appt_datetime"].replace(tzinfo=UTC)
        assignment = rx.get_assignment()
        next_id = get_next_value(stock_request_item_model_cls._meta.label_lower)
        request_item_identifier = f"{next_id:06d}"
        obj = stock_request_item_model_cls(
            stock_request=stock_request,
            request_item_identifier=request_item_identifier,
            registered_subject=registered_subject,
            visit_code=visit_code,
            visit_code_sequence=visit_code_sequence,
            rx=rx,
            appt_datetime=appt_datetime,
            assignment=assignment,
            user_created=user_created,
            created=now,
        )
        data.append(obj)
    if data:
        if bulk_create:
            stock_request_item_model_cls.objects.bulk_create(data)
        else:
            for obj in data:
                obj.save()
        stock_request.item_count = len(data)
        stock_request.save(update_fields=["item_count"])


__all__ = ["bulk_create_stock_request_items"]



from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

from edc_model import model_exists_or_raise


def raise_if_crf_does_not_exist(subject_visit, model: str) -> None:
    model_cls = django_apps.get_model(model)
    try:
        model_exists_or_raise(subject_visit=subject_visit, model_cls=model_cls)
    except ObjectDoesNotExist:
        complete = _("Complete")
        crf_first = _("CRF first")
        verbose_name = model_cls._meta.verbose_name
        err_message = format_lazy(
            "{complete} {verbose_name} {crf_first}",
            complete=complete,
            crf_first=crf_first,
            verbose_name=verbose_name,
        )
        raise forms.ValidationError(err_message)


class HasCrfChecker:
    def __init__(
        self,
        app_name: str,
        subject_visit_model_cls,
    ):
        self.app_name = app_name
        self.subject_visit_model_cls = subject_visit_model_cls
        self.models = []
        for model in django_apps.get_app_config(self.app_name).get_models():
            if model._meta.label_lower.split(".")[1].startswith("historical"):
                continue
            if "subject_visit" in [f.name for f in model._meta.get_fields()]:
                self.models.append(model)

    def has_crfs(
        self,
        subject_identifier,
        visit_code: str,
        visit_code_sequence: int,
    ):
        subject_visit = None
        try:
            subject_visit = self.subject_visit_model_cls.objects.get(
                subject_identifier=subject_identifier,
                visit_code=visit_code,
                visit_code_sequence=visit_code_sequence,
            )
        except ObjectDoesNotExist:
            pass
        except MultipleObjectsReturned:
            raise MultipleObjectsReturned("Specify the visit_code and visit_code_sequence")
        else:
            for model in self.models:
                if model.objects.filter(subject_visit=subject_visit).exists():
                    return subject_visit, True
        return subject_visit, False



import re
from uuid import uuid4

from django.apps import apps as django_apps
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import models

from edc_constants.constants import UUID_PATTERN

from .exceptions import IdentifierError
from .subject_identifier import SubjectIdentifier


class NonUniqueSubjectIdentifierFieldMixin(models.Model):
    """An internal model mixin providing a non-unique subject
    identifier field.
    """

    subject_identifier = models.CharField(max_length=50)

    class Meta:
        abstract = True
        indexes = (models.Index(fields=["subject_identifier"]),)


class UniqueSubjectIdentifierFieldMixin(models.Model):
    """An internal model mixin providing a unique subject identifier
    field.
    """

    subject_identifier = models.CharField(max_length=50, unique=True)

    class Meta:
        abstract = True


class SubjectIdentifierAdditionalFieldsModelMixin(models.Model):
    """An internal model mixin providing additional fields to used
    with the subject identifier field.
    """

    subject_identifier_as_pk = models.UUIDField(default=uuid4, editable=False)

    subject_identifier_aka = models.CharField(
        verbose_name="Subject Identifier a.k.a",
        max_length=50,
        default="",
        editable=False,
        help_text="track a previously allocated identifier.",
    )

    class Meta:
        abstract = True


class SubjectIdentifierMethodsModelMixin(models.Model):
    """An internal model mixin to add a unique subject identifier
    field and fill it with a unique value for new instances.
    """

    subject_identifier_cls = SubjectIdentifier

    def save(self, *args, **kwargs):
        if not self.id:
            self.subject_identifier = self.update_subject_identifier_on_save()
        super().save(*args, **kwargs)

    def update_subject_identifier_on_save(self) -> str | models.CharField:
        """Returns a subject_identifier if not already set."""
        if not self.subject_identifier or re.match(UUID_PATTERN, self.subject_identifier):
            self.subject_identifier = self.get_or_create_identifier()
        return self.subject_identifier

    @property
    def registered_subject_model_cls(self):
        """Returns the registered subject model class."""
        return django_apps.get_model("edc_registration.registeredsubject")

    def get_or_create_identifier(self) -> str:
        """Returns a subject identifier either by retrieving and
        exisiting "subject identifier from RegisteredSubject or
        creating a new and unique "subject" identifier.
        """
        try:
            subject_identifier = self.registered_subject.subject_identifier
        except ObjectDoesNotExist:
            subject_identifier = self.make_new_identifier()
        return subject_identifier

    def make_new_identifier(self) -> str:
        """Returns a new and unique identifier.

        Override this if needed.
        """
        subject_identifier = self.subject_identifier_cls(
            identifier_type="subject",
            requesting_model=self._meta.label_lower,
            site=self.site,
        )
        return subject_identifier.identifier

    @property
    def registered_subject(self):
        """Returns a registered subject instance.

        Override this if your query options are different.

        Will raise ObjectDoesNotExist if this is a new instance.
        """
        try:
            obj = self.registered_subject_model_cls.objects.get(identity=self.identity)
        except MultipleObjectsReturned as e:
            raise IdentifierError(
                "Cannot lookup a unique RegisteredSubject instance. "
                f"Identity {self.identity} is not unique. Got {e}"
            ) from e
        return obj

    class Meta:
        abstract = True


class UniqueSubjectIdentifierModelMixin(
    UniqueSubjectIdentifierFieldMixin,
    SubjectIdentifierAdditionalFieldsModelMixin,
    SubjectIdentifierMethodsModelMixin,
    models.Model,
):
    """A model mixin for concrete models requiring a unique subject
    identifier field and corresponding fields and methods.
    """

    def get_subject_identifier(self):
        """Returns the subject_identifier"""
        return self.subject_identifier

    def get_subject_identifier_as_pk(self):
        """Returns the subject_identifier_as_pk"""
        return self.subject_identifier_as_pk

    class Meta:
        abstract = True


class NonUniqueSubjectIdentifierModelMixin(
    NonUniqueSubjectIdentifierFieldMixin,
    SubjectIdentifierAdditionalFieldsModelMixin,
    SubjectIdentifierMethodsModelMixin,
    models.Model,
):
    """A model mixin for concrete models requiring a non-unique
    subject identifier field and corresponding field and methods.
    """

    def get_subject_identifier(self):
        """Returns the subject_identifier"""
        return self.subject_identifier

    def get_subject_identifier_as_pk(self):
        """Returns the subject_identifier_as_pk"""
        return self.subject_identifier_as_pk

    class Meta(NonUniqueSubjectIdentifierFieldMixin.Meta):
        abstract = True



# Generated by Django 3.2.13 on 2022-09-29 14:42

from django.db import migrations, models

import edc_identifier.managers
import edc_sites.models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_offstudy", "0015_auto_20220925_0032"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="subjectoffstudy",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AddField(
            model_name="historicalsubjectoffstudy",
            name="report_datetime",
            field=models.DateTimeField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name="subjectoffstudy",
            name="report_datetime",
            field=models.DateTimeField(editable=False, null=True),
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_refusal", "0012_alter_historicalsubjectrefusal_comment_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
import edc_model.validators.date
import edc_protocol.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_visit_tracking", "0009_alter_historicalsubjectvisit_comments_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="Date and time of this report",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report Date",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="Date and time of this report",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report Date",
            ),
        ),
    ]



from .crf_metadata_getter import CrfMetadataGetter
from .metadata import (
    CreatesMetadataError,
    Creator,
    DeleteMetadataError,
    Destroyer,
    Metadata,
)
from .metadata_getter import MetadataGetter
from .requisition_metadata_getter import RequisitionMetadataGetter



from django.contrib import admin

from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from .admin_site import edc_locator_admin
from .forms import SubjectLocatorForm
from .modeladmin_mixins import SubjectLocatorModelAdminMixin
from .models import SubjectLocator


@admin.register(SubjectLocator, site=edc_locator_admin)
class SubjectLocatorAdmin(
    SubjectLocatorModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = SubjectLocatorForm



from django.apps import AppConfig
from django.core.management.color import color_style

from edc_list_data.load_model_data import load_model_data

from .load_list_data import load_list_data

style = color_style()


class PreloadData:
    def __init__(
        self,
        list_data: dict[str, list[tuple[str | int, str]]],
        model_data: dict | None = None,
        list_data_model_name: str | None = None,
        apps: AppConfig = None,
    ) -> None:
        self.list_data = list_data
        self.model_data = model_data or {}
        self.item_count = 0
        if self.list_data:
            self.item_count += self.load_list_data(model_name=list_data_model_name, apps=apps)
        if self.model_data:
            self.item_count += self.load_model_data()

    def load_list_data(self, model_name: str, apps: AppConfig | None = None) -> int:
        return load_list_data(self.list_data, model_name=model_name, apps=apps)

    def load_model_data(self, apps: AppConfig | None = None) -> int:
        return load_model_data(self.model_data, apps=apps)



from django import forms

from edc_constants.constants import YES

from ..utils import calculate_avg_bp, has_severe_htn


class BloodPressureFormValidatorMixin:
    """Coupled with BloodPressureModelMixin"""

    @staticmethod
    def raise_on_avg_blood_pressure_suggests_severe_htn(
        use_avg=None, severe_htn_field_name=None, errmsg=None, **kwargs
    ):
        """Raise if BP is >= 180/110, See settings"""
        severe_htn_field_name = severe_htn_field_name or "severe_htn"
        severe_htn_response = kwargs.get(severe_htn_field_name)
        errmsg = {
            severe_htn_field_name: (errmsg or "Invalid. Patient has severe hypertension")
        }
        use_avg = True if use_avg is None else False  # noqa: SIM210
        avg_sys, avg_dia = calculate_avg_bp(use_av=use_avg, **kwargs)
        if has_severe_htn(sys=avg_sys, dia=avg_dia) and severe_htn_response != YES:
            raise forms.ValidationError(errmsg)

    @staticmethod
    def raise_on_systolic_lt_diastolic_bp(
        sys_field: str | None = None, dia_field: str | None = None, **kwargs
    ) -> None:
        """Raise if systolic BP is < diastolic BP."""
        sys_field = sys_field or "sys_blood_pressure"
        dia_field = dia_field or "dia_blood_pressure"
        sys_response = kwargs.get(sys_field)
        dia_response = kwargs.get(dia_field)
        if sys_response and dia_response and sys_response < dia_response:
            raise forms.ValidationError(
                {dia_field: "Invalid. Diastolic must be less than systolic."}
            )



from datetime import datetime
from zoneinfo import ZoneInfo

from django.core.exceptions import ValidationError
from django.utils import timezone

from edc_utils import formatted_datetime

from .research_protocol_config import ResearchProtocolConfig


def date_not_before_study_start(value):
    if value:
        protocol_config = ResearchProtocolConfig()
        value_utc = datetime(*[*value.timetuple()][0:6], tzinfo=ZoneInfo("UTC"))
        if value_utc < protocol_config.study_open_datetime:
            opened = formatted_datetime(
                timezone.localtime(protocol_config.study_open_datetime)
            )
            got = formatted_datetime(timezone.localtime(value_utc))
            raise ValidationError(
                f"Invalid date. Study opened on {opened}. Got {got}. "
                f"See edc_protocol.AppConfig."
            )


def datetime_not_before_study_start(value_datetime):
    if value_datetime:
        protocol_config = ResearchProtocolConfig()
        value_utc = value_datetime.astimezone(ZoneInfo("UTC"))
        if value_utc < protocol_config.study_open_datetime:
            opened = formatted_datetime(
                timezone.localtime(protocol_config.study_open_datetime)
            )
            got = formatted_datetime(timezone.localtime(value_utc))
            raise ValidationError(
                f"Invalid date/time. Study opened on {opened}. Got {got}."
                f"See edc_protocol.AppConfig."
            )



from __future__ import annotations

from datetime import date, datetime
from decimal import Decimal
from typing import TYPE_CHECKING, Any
from zoneinfo import ZoneInfo

from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.utils import timezone

from edc_constants.constants import NEW
from edc_reportable.models import ReferenceRangeCollection
from edc_reportable.units import EGFR_UNITS, PERCENT
from edc_reportable.utils import get_grade_for_value
from edc_utils.age import age

from .calculators import EgfrCkdEpi, EgfrCockcroftGault, egfr_percent_change
from .get_drop_notification_model import get_egfr_drop_notification_model_cls

if TYPE_CHECKING:
    from edc_visit_tracking.typing_stubs import RelatedVisitProtocol


class EgfrError(Exception):
    pass


class Egfr:
    calculators: dict = {"ckd-epi": EgfrCkdEpi, "cockcroft-gault": EgfrCockcroftGault}  # noqa: RUF012

    def __init__(
        self,
        baseline_egfr_value: Decimal | float | None = None,
        gender: str | None = None,
        ethnicity: str | None = None,
        age_in_years: int | None = None,
        dob: date | None = None,
        weight_in_kgs: Decimal | float | None = None,
        report_datetime: datetime | None = None,
        creatinine_value: Decimal | float | None = None,
        creatinine_units: str | None = None,
        formula_name: str | None = None,
        value_threshold: Decimal | float | None = None,
        percent_drop_threshold: Decimal | float | None = None,
        reference_range_collection: ReferenceRangeCollection | None = None,
        reference_range_collection_name: str | None = None,
        calling_crf: Any | None = None,
        related_visit: RelatedVisitProtocol | None = None,
        assay_datetime: datetime | None = None,
        egfr_drop_notification_model: str | None = None,
    ):
        self._egfr_value: Decimal | None = None
        self._egfr_grade = None
        self._egfr_drop_value: Decimal | None = None
        self._egfr_drop_grade = None
        self.assay_date: date | None = None
        self.related_visit = None

        self.baseline_egfr_value = baseline_egfr_value
        if formula_name not in self.calculators:
            raise EgfrError(
                f"Invalid formula_name. Expected one of {list(self.calculators.keys())}. "
                f"Got {formula_name}."
            )
        self.calculator_cls = self.calculators.get(formula_name)
        self.age_in_years = age_in_years
        self.dob = dob
        self.weight_in_kgs = weight_in_kgs
        self.egfr_drop_notification_model = egfr_drop_notification_model
        self.egfr_drop_units = PERCENT
        self.egfr_units = EGFR_UNITS
        self.ethnicity = ethnicity
        self.gender = gender
        if not reference_range_collection:
            try:
                self.reference_range_collection = ReferenceRangeCollection.objects.get(
                    name=reference_range_collection_name
                )
            except ObjectDoesNotExist as e:
                raise ObjectDoesNotExist(f"{e} Got {reference_range_collection_name}") from e
        else:
            self.reference_range_collection = reference_range_collection
        self.report_datetime = report_datetime
        self.value_threshold = value_threshold
        self.percent_drop_threshold = percent_drop_threshold

        if self.dob:
            self.age_in_years = age(
                born=self.dob,
                reference_dt=self.report_datetime.astimezone(ZoneInfo("UTC")),
            ).years
        elif not self.dob and self.age_in_years:
            self.dob = (
                self.report_datetime.astimezone(ZoneInfo("UTC"))
                - relativedelta(years=self.age_in_years)
            ).date()
        else:
            raise EgfrError("Expected `age_in_years` or `dob`. Got None for both.")

        if calling_crf:
            self.creatinine_units = calling_crf.creatinine_units
            self.creatinine_value = calling_crf.creatinine_value
            self.percent_drop_threshold = calling_crf.percent_drop_threshold
            self.related_visit = calling_crf.related_visit
            self.report_datetime = calling_crf.report_datetime
            self.assay_date = calling_crf.assay_datetime.astimezone(ZoneInfo("UTC")).date()
        else:
            self.creatinine_units = creatinine_units
            self.creatinine_value = creatinine_value
            self.percent_drop_threshold = percent_drop_threshold
            self.related_visit = related_visit
            self.report_datetime = report_datetime
            if assay_datetime:
                self.assay_date = assay_datetime.astimezone(ZoneInfo("UTC")).date()

        if self.percent_drop_threshold is not None and self.percent_drop_threshold < 1.0:
            raise EgfrError(
                "Attr `percent_drop_threshold` should be a percentage. "
                f"Got {self.percent_drop_threshold}"
            )

        self.on_value_threshold_reached()

        self.on_percent_drop_threshold_reached()

    def on_value_threshold_reached(self) -> None:
        """A hook to respond if egfr value is at or beyond the value
        threshold.
        """
        pass

    def on_percent_drop_threshold_reached(self) -> None:
        """A hook to respond if egfr percent drop from baseline
        is at or beyond the percent drop threshold.
        """
        if (
            self.egfr_drop_value
            and self.percent_drop_threshold is not None
            and self.egfr_drop_value >= self.percent_drop_threshold
        ):
            self.create_or_update_egfr_drop_notification()

    @property
    def egfr_value(self) -> float:
        if self._egfr_value is None:
            self._egfr_value = self.calculator_cls(
                gender=self.gender,
                ethnicity=self.ethnicity,
                age_in_years=self.age_in_years,
                creatinine_value=self.creatinine_value,
                creatinine_units=self.creatinine_units,
                weight=self.get_weight_in_kgs(),
            ).value
        return self._egfr_value

    @property
    def egfr_grade(self) -> int | None:
        if self._egfr_grade is None:
            grading_data, _ = get_grade_for_value(
                self.reference_range_collection,
                value=self.egfr_value,
                label="egfr",
                gender=self.gender,
                dob=self.dob,
                report_datetime=self.report_datetime,
                units=self.egfr_units,
                age_units="years",
            )
            if grading_data:
                self._egfr_grade = grading_data.grade
        return self._egfr_grade

    @property
    def egfr_drop_value(self) -> float:
        if self._egfr_drop_value is None:
            if self.baseline_egfr_value:
                egfr_drop_value = egfr_percent_change(
                    float(self.egfr_value), float(self.baseline_egfr_value)
                )
            else:
                egfr_drop_value = 0.0000
            self._egfr_drop_value = max(egfr_drop_value, 0.0)
        return self._egfr_drop_value

    @property
    def egfr_drop_grade(self) -> int | None:
        if self._egfr_drop_grade is None:
            grading_data, _ = get_grade_for_value(
                self.reference_range_collection,
                value=self.egfr_drop_value,
                label="egfr_drop",
                gender=self.gender,
                dob=self.dob,
                report_datetime=self.report_datetime,
                units=self.egfr_drop_units,
                age_units="years",
            )
            if grading_data:
                self._egfr_drop_grade = grading_data.grade
        return self._egfr_drop_grade

    def get_weight_in_kgs(self) -> float | None:
        return self.weight_in_kgs

    def create_or_update_egfr_drop_notification(self):
        """Creates or updates the `eGFR notification model`"""
        with transaction.atomic():
            try:
                obj = self.egfr_drop_notification_model_cls.objects.get(
                    subject_visit__id=self.related_visit.id
                )
            except ObjectDoesNotExist:
                obj = self.egfr_drop_notification_model_cls.objects.create(
                    subject_visit_id=self.related_visit.id,
                    report_datetime=self.report_datetime,
                    egfr_value=self.egfr_value,
                    creatinine_date=self.assay_date,
                    creatinine_value=self.creatinine_value,
                    creatinine_units=self.creatinine_units,
                    weight=self.get_weight_in_kgs(),
                    egfr_percent_change=self.egfr_drop_value,
                    report_status=NEW,
                    site_id=self.related_visit.site.id,
                )
            else:
                obj.egfr_value = self.egfr_value
                obj.creatinine_value = self.creatinine_value
                obj.weight = self.get_weight_in_kgs()
                obj.egfr_percent_change = self.egfr_drop_value
                obj.creatinine_date = self.assay_date
                obj.modified = timezone.now()
                obj.save()
        obj.refresh_from_db()
        return obj

    @property
    def egfr_drop_notification_model_cls(self):
        return get_egfr_drop_notification_model_cls()



from django.conf import settings

from .constants import (
    ABNORMAL,
    CANCELLED,
    CLOSED,
    COMPLETE,
    FEMALE,
    INCOMPLETE,
    LIVE,
    MALE,
    NEW,
    NO,
    NOT_APPLICABLE,
    OPEN,
    OTHER,
    PENDING,
    TBD,
    TEST,
    UNKNOWN,
    YES,
)


def constants(request) -> dict:
    return dict(
        ABNORMAL=ABNORMAL,
        CANCELLED=CANCELLED,
        CLOSED=CLOSED,
        COMPLETE=COMPLETE,
        DEBUG=settings.DEBUG,
        FEMALE=FEMALE,
        INCOMPLETE=INCOMPLETE,
        LIVE=LIVE,
        MALE=MALE,
        NEW=NEW,
        NO=NO,
        NOT_APPLICABLE=NOT_APPLICABLE,
        OPEN=OPEN,
        OTHER=OTHER,
        PENDING=PENDING,
        SITE_ID=settings.SITE_ID,
        TBD=TBD,
        TEST=TEST,
        UNKNOWN=UNKNOWN,
        YES=YES,
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..constants import REQUISITION
from ..metadata_updater import MetadataUpdater
from .requisition_metadata_handler import RequisitionMetadataHandler

if TYPE_CHECKING:
    from edc_lab.models import Panel


class RequisitionMetadataError(Exception):
    pass


class RequisitionMetadataUpdater(MetadataUpdater):
    """A class to update a subject's requisition metadata given
    the visit, target model name, panel and desired entry status.
    """

    metadata_handler_cls: RequisitionMetadataHandler = RequisitionMetadataHandler
    metadata_category: str = REQUISITION
    metadata_model: str = "edc_metadata.requisitionmetadata"

    def __init__(self, source_panel: Panel = None, **kwargs):
        super().__init__(**kwargs)
        self.source_panel = source_panel

    @property
    def metadata_handler(self):
        return self.metadata_handler_cls(
            metadata_model=self.metadata_model,
            model=self.source_model,
            related_visit=self.related_visit,
            panel=self.source_panel,
            allow_create=self.allow_create,
        )

    @property
    def source_model_options(self) -> dict[str, Any]:
        """Returns a dictionary of query options to filter for, or
        get, the SubjectRequisition model instance.
        """
        return dict(subject_visit_id=self.related_visit.id, panel__name=self.source_panel.name)



# Generated by Django 5.2.3 on 2025-07-08 20:37
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from sequences.models import Sequence


def update_sequences(apps, schema_editor):
    try:
        obj = Sequence.objects.get(name="edc_pharmacy.stocktransferconfirmation")
    except ObjectDoesNotExist:
        pass
    else:
        obj.name = "edc_pharmacy.confirmationatsite"
        obj.save()
    try:
        obj = Sequence.objects.get(name="edc_pharmacy.stocktransferconfirmationitem")
    except ObjectDoesNotExist:
        pass
    else:
        obj.name = "edc_pharmacy.confirmationatsiteitem"
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("sequences", "0002_alter_sequence_last"),
        ("edc_pharmacy", "0085_alter_confirmationatsite_options_and_more"),
    ]

    operations = [migrations.RunPython(update_sequences)]



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from edc_utils import to_local

from ..ordered_collection import OrderedCollection

if TYPE_CHECKING:
    from ..visit import Visit


class VisitCollectionError(Exception):
    pass


class VisitCollection(OrderedCollection):
    key: str = "code"
    ordering_attr: str = "timepoint"

    def get(self, visit_code: str) -> Visit:
        """Return a visit for the given visit_code or raise"""
        value = super().get(visit_code)
        if value is None:
            raise VisitCollectionError(
                f"Unknown visit. Check the visit schedule. Got visit_code={visit_code}"
            )
        return value

    def timepoint_dates(self, dt: datetime) -> dict:
        """Returns an ordered dictionary of visit dates calculated
        relative to the first visit.
        """
        timepoint_dates = {}
        for visit in self.values():
            try:
                timepoint_datetime = to_local(dt) + visit.rbase
            except TypeError as e:
                raise VisitCollectionError(
                    f"Invalid visit.rbase. visit.rbase={visit.rbase}. See {visit!r}. Got {e}."
                ) from e
            else:
                visit.timepoint_datetime = timepoint_datetime
            timepoint_dates.update({visit: visit.timepoint_datetime})

        last_dte = None
        for dte in timepoint_dates.values():
            if not last_dte:
                last_dte = dte
                continue
            if dte and last_dte and not dte > last_dte:
                raise VisitCollectionError(
                    "Wait! timepoint datetimes are not in sequence. "
                    f"Check visit.rbase in your visit collection. See {self}."
                )

        return timepoint_dates

    @property
    def timepoints(self) -> dict:
        timepoints = {}
        for visit in self.values():
            timepoints.update({visit: visit.timepoint})
        return timepoints



from edc_model_wrapper import ModelWrapper

from edc_lab.models import Box


class BoxModelWrapper(ModelWrapper):
    model_cls = Box
    next_url_name = "pack_listboard_url"

    @property
    def human_readable_identifier(self):
        return self.object.human_readable_identifier



from __future__ import annotations

import dataclasses
import sys
from copy import deepcopy
from typing import TYPE_CHECKING, Any

from django.apps import apps as django_apps
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.core.handlers.wsgi import WSGIRequest as BaseWSGIRequest
from django.core.management.color import color_style
from django.utils.module_loading import import_module, module_has_submodule

from edc_auth.utils import user_has_change_perms
from edc_constants.constants import OTHER
from edc_model_admin.utils import add_to_messages_once

from .exceptions import InvalidSiteForUser
from .single_site import SingleSite
from .utils import (
    get_message_text,
    get_site_model_cls,
    has_profile_or_raise,
    insert_into_domain,
)

if TYPE_CHECKING:
    from django.contrib.auth.models import User
    from django.contrib.sites.models import Site

    class WSGIRequest(BaseWSGIRequest):
        site: Site


__all__ = [
    "AlreadyRegistered",
    "AlreadyRegisteredDomain",
    "AlreadyRegisteredName",
    "SiteDoesNotExist",
    "SiteNotRegistered",
    "SitesCheckError",
    "SitesError",
    "get_autodiscover_sites",
    "get_default_country",
    "get_default_country_code",
    "get_default_domain",
    "get_insert_uat_subdomain",
    "get_register_default_site",
    "sites",
]


class SiteDoesNotExist(Exception):  # noqa: N818
    pass


class AlreadyRegistered(Exception):  # noqa: N818
    pass


class AlreadyRegisteredName(Exception):  # noqa: N818
    pass


class AlreadyRegisteredDomain(Exception):  # noqa: N818
    pass


class SiteNotRegistered(Exception):  # noqa: N818
    pass


class SitesCheckError(Exception):
    pass


class SitesError(Exception):
    pass


app_name: str = getattr(settings, "APP_NAME", "edc")


def get_register_default_site() -> bool:
    return getattr(settings, "EDC_SITES_REGISTER_DEFAULT", False)


def get_default_country() -> str:
    return getattr(settings, "EDC_SITES_DEFAULT_COUNTRY", "botswana")


def get_default_country_code() -> str:
    return getattr(settings, "EDC_SITES_DEFAULT_COUNTRY_CODE", "bw")


def get_default_domain() -> str:
    return getattr(settings, "EDC_SITES_DEFAULT_DOMAIN", "localhost")


def get_insert_uat_subdomain() -> str | None:
    return getattr(settings, "EDC_SITES_UAT_DOMAIN", None)


def get_autodiscover_sites():
    return getattr(settings, "EDC_SITES_AUTODISCOVER_SITES", True)


class Sites:
    uat_subdomain = "uat"

    def __init__(self):
        self.loaded = False
        self._registry = {}
        if get_register_default_site():
            self.loaded = True
            site_id = int(settings.SITE_ID)
            self._registry: dict[int, SingleSite] = {
                site_id: SingleSite(
                    site_id,
                    settings.APP_NAME,
                    country=get_default_country(),
                    country_code=get_default_country_code(),
                    domain=get_default_domain(),
                    title="what a site",
                )
            }

    def __repr__(self):
        return f"{self.__class__}(loaded={self.loaded})"

    def __str__(self):
        return f"loaded={self.loaded}, registry={self._registry}"

    def initialize(self, initialize_site_model=False):
        """Initialize the registry.

        This is for tests where you are manipulating the ``sites``
        registry. If you do this after the  ``post-migrate`` signal
        is called you will also have to initialize the Site model.
        """
        if initialize_site_model:
            for obj in get_site_model_cls().objects.all():
                try:
                    obj.siteprofile.delete()
                except ObjectDoesNotExist:
                    break
            get_site_model_cls().objects.all().delete()
        self.__init__()

    def register(self, *single_sites: SingleSite):
        if not self.loaded:
            self._registry = {}
            self.loaded = True
        if "makemigrations" not in sys.argv:
            for single_site in single_sites:
                if get_insert_uat_subdomain():
                    domain = insert_into_domain(single_site.domain, self.uat_subdomain)
                    single_site = dataclasses.replace(single_site, domain=domain)  # noqa: PLW2901

                if single_site.site_id in self._registry:
                    raise AlreadyRegistered(f"Site already registered. Got `{single_site}`.")
                if single_site.name in [s.name for s in self._registry.values()]:
                    raise AlreadyRegisteredName(
                        f"Site with this name is already registered. Got `{single_site}`."
                    )
                if single_site.domain in [s.domain for s in self._registry.values()]:
                    raise AlreadyRegisteredDomain(
                        f"Site with this domain is already registered. Got `{single_site}`."
                    )
                self._registry.update({single_site.site_id: single_site})

    def get(self, site_id: int) -> SingleSite:
        """Returns a SingleSite instance for this site_id or
        raises.

        If the reqistry is empty, check you have created a `sites.py`
        and called `register`.

        If you are just running tests, you may only need to set
        ``settings.EDC_SITES_REGISTER_DEFAULT=True``.
        """
        if site_id not in self._registry:
            site_ids = [s.site_id for s in self.all(aslist=True)]
            if not site_ids:
                msg = "In fact, no sites have been registered!"
            else:
                msg = f"Expected one of {[s.site_id for s in self.all(aslist=True)]}."
            raise SiteNotRegistered(
                f"Site not registered. {msg} See {self!r}. Got `{site_id}`."
            )
        return self._registry.get(site_id)

    def get_by_attr(self, attrname: str, value: Any) -> SingleSite:
        for single_site in self._registry.values():
            if getattr(single_site, attrname) == value:
                return single_site
        raise SiteDoesNotExist(f"No site exists with `{attrname}`==`{value}`.")

    def all(self, aslist: bool | None = None) -> dict[int, SingleSite] | list[SingleSite]:
        if aslist:
            return list(self._registry.values())
        return self._registry

    @property
    def countries(self) -> list[str]:
        return (
            list(set([single_site.country for single_site in self._registry.values()])) or []
        )

    def get_by_country(
        self, country: str, aslist: bool | None = None
    ) -> dict[int, SingleSite] | list[SingleSite]:
        if aslist:
            return [
                single_site
                for single_site in self.all(aslist=True)
                if single_site.country == country
            ]

        return {
            site_id: single_site
            for site_id, single_site in self._registry.items()
            if single_site.country == country
        }

    def get_site_ids_for_user(
        self,
        request: WSGIRequest | None = None,
        user: User | None = None,
        site_id: int | None = None,
    ) -> list[int]:
        """Returns a list of site ids for this user, including the
        current site.
        """
        if request:
            user = request.user
            site_id = request.site.id
        return [
            site_id,
            *self.get_view_only_site_ids_for_user(request=request, user=user, site_id=site_id),
        ]

    @staticmethod
    def get_view_only_site_ids_for_user(
        request: WSGIRequest | None = None,
        user: User | None = None,
        site_id: int | None = None,
    ) -> list[int]:
        """Returns a list of any sites the user may have view
        access to, not including the current.

        Checks for userprofile.is_multisite_viewer and
        confirms user does not have `add`, `change` or `delete`
        perms to any resources.
        """
        if request:
            user = request.user
            site_id = request.site.id
        site_id = sites.get(site_id).site_id
        has_profile_or_raise(user)
        sites.site_in_profile_or_raise(user=user, site_id=site_id)
        # now check for special view codename from user account
        site_ids = []
        if user.userprofile.is_multisite_viewer:
            if user_has_change_perms(user=user):
                if request:
                    add_to_messages_once(
                        request,
                        messages.ERROR,
                        get_message_text(messages.ERROR),
                    )
            else:
                site_ids = [s.id for s in user.userprofile.sites.all() if s.id != site_id]
                if request:
                    add_to_messages_once(
                        request, messages.WARNING, get_message_text(messages.WARNING)
                    )
        return site_ids

    def user_may_view_other_sites(
        self,
        request: WSGIRequest = None,
        user: User | None = None,
        site_id: int | None = None,
    ) -> list[int]:
        return self.get_view_only_site_ids_for_user(
            request=request,
            user=user,
            site_id=site_id,
        )

    @staticmethod
    def site_in_profile_or_raise(user: User, site_id: int) -> None:
        """Raises if user does not have site in their UserProfile."""
        try:
            user.userprofile.sites.get(id=site_id).id  # noqa: B018
        except ObjectDoesNotExist as e:
            errmsg = (
                "User is not configured to access this site. See also UserProfile. "
                f"Got {site_id}."
            )
            raise InvalidSiteForUser(errmsg) from e

    def get_language_choices_tuple(
        self, site: Site | None = None, site_id: int | None = None, other=None
    ) -> tuple | None:
        """Returns a choices tuple of languages from the site object to
        be used on the `languages` modelform field.

        See also: SingleSite and SiteModelAdminMixin.
        """
        site_id = getattr(site, "id", site_id)
        single_site = self.get(site_id)
        languages = single_site.languages
        if other:
            languages.update({OTHER: "Other"})
        return tuple((k, v) for k, v in languages.items())

    @staticmethod
    def get_current_site_obj(request: WSGIRequest | None = None) -> Site:
        if request:
            return request.site
        return get_site_model_cls().objects.get_current()

    def get_current_site(self, request: WSGIRequest | None = None) -> SingleSite:
        if request:
            return self.get(request.site.id)
        return self.get(get_site_model_cls().objects.get_current().id)

    def get_current_country(self, request: WSGIRequest | None = None) -> str:
        single_site = self.get_current_site(request)
        return single_site.country

    @staticmethod
    def autodiscover(module_name=None, verbose=True):
        """Autodiscovers query rule classes in the sites.py file of
        any INSTALLED_APP.
        """
        module_name = module_name or "sites"
        writer = sys.stdout.write if verbose else lambda x: x
        style = color_style()
        writer(f" * checking for {module_name} (edc_sites)...\n")
        for app in django_apps.app_configs:
            try:
                mod = import_module(app)
                try:
                    before_import_registry = deepcopy(sites._registry)
                    import_module(f"{app}.{module_name}")
                    writer(f"   - registered '{module_name}' from '{app}'\n")
                except SitesError as e:
                    writer(f"   - loading {app}.{module_name} ... ")
                    writer(style.ERROR(f"ERROR! {e}\n"))
                except ImportError as e:
                    sites._registry = before_import_registry
                    if module_has_submodule(mod, module_name):
                        raise SitesError(str(e)) from e
            except ImportError:
                pass


sites = Sites()



from django.apps import AppConfig as DjangoAppConfig
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_list_data"
    verbose_name = "Edc List Data"



# Generated by Django 4.2.1 on 2023-07-07 19:25

from django.db import migrations

import edc_action_item.models.action_item


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_action_item",
            "0032_alter_actionitem_options_alter_actiontype_options_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="actionitem",
            managers=[
                ("objects", edc_action_item.models.action_item.ActionItemManager()),
                ("on_site", edc_action_item.models.action_item.CurrentSiteManager()),
            ],
        ),
    ]



import socket


def get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(("10.254.254.254", 1))
        ip = s.getsockname()[0]
    except Exception:
        ip = "127.0.0.1"
    finally:
        s.close()
    return ip


def scan_network(ip_range):
    open_printers = []
    for ip in ip_range:
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.settimeout(1)  # Timeout in seconds
            result = sock.connect_ex((ip, 9100))  # Port 9100 is common for network printers
            if result == 0:
                open_printers.append(ip)
            sock.close()
        except Exception as e:
            print(f"Error scanning {ip}: {e}")
    return open_printers


def get_ip_range(local_ip):
    ip_parts = local_ip.split(".")
    base_ip = ".".join(ip_parts[:3]) + "."
    return [base_ip + str(i) for i in range(1, 255)]


def main():
    local_ip = get_local_ip()
    ip_range = get_ip_range(local_ip)
    printers = scan_network(ip_range)
    if printers:
        print("Found network printers at the following IP addresses:")
        for printer in printers:
            print(printer)
    else:
        print("No network printers found.")


if __name__ == "__main__":
    main()



# Generated by Django 2.0 on 2018-01-02 22:00

import _socket
import django.contrib.sites.managers
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.uuid_auto_field
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.encrypted_text_field
import django_revision.revision_field
from django.conf import settings
from django.db import migrations, models

import edc_model.validators.phone
import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalSubjectLocator",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "subject_identifier",
                    models.CharField(
                        db_index=True, max_length=50, verbose_name="Subject Identifier"
                    ),
                ),
                ("tracking_identifier", models.CharField(db_index=True, max_length=30)),
                ("action_identifier", models.CharField(max_length=25, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "may_call",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Has the participant given permission <b>to contacted by telephone or cell</b> by study staff for follow-up purposes during the study?",
                    ),
                ),
                (
                    "may_visit_home",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Has the participant given permission for studystaff <b>to make home visits</b> for follow-up purposes?",
                    ),
                ),
                (
                    "may_sms",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="Has the participant given permission <b>to be contacted by SMS</b> by study staff for follow-up purposes during the study?",
                    ),
                ),
                (
                    "mail_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Mailing address ",
                    ),
                ),
                (
                    "physical_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Physical address with detailed description",
                    ),
                ),
                (
                    "subject_cell",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number",
                    ),
                ),
                (
                    "subject_cell_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number (alternate)",
                    ),
                ),
                (
                    "subject_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone",
                    ),
                ),
                (
                    "subject_phone_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone (alternate)",
                    ),
                ),
                (
                    "subject_work_place",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=250,
                        null=True,
                        verbose_name="Name and location of work place",
                    ),
                ),
                (
                    "subject_work_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Work contact number ",
                    ),
                ),
                (
                    "may_contact_indirectly",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="For example a partner, spouse, family member, neighbour ...",
                        max_length=25,
                        verbose_name="Has the participant given permission for study staff <b>to contact anyone else</b> for follow-up purposes during the study?",
                    ),
                ),
                (
                    "indirect_contact_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Full names of the contact person",
                    ),
                ),
                (
                    "indirect_contact_relation",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Relationship to participant",
                    ),
                ),
                (
                    "indirect_contact_physical_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Full physical address ",
                    ),
                ),
                (
                    "indirect_contact_cell",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number",
                    ),
                ),
                (
                    "indirect_contact_cell_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number (alternative)",
                    ),
                ),
                (
                    "indirect_contact_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone number",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical ",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
        ),
        migrations.CreateModel(
            name="SubjectLocator",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "subject_identifier",
                    models.CharField(
                        max_length=50, unique=True, verbose_name="Subject Identifier"
                    ),
                ),
                ("tracking_identifier", models.CharField(max_length=30, unique=True)),
                ("action_identifier", models.CharField(max_length=25, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "may_call",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Has the participant given permission <b>to contacted by telephone or cell</b> by study staff for follow-up purposes during the study?",
                    ),
                ),
                (
                    "may_visit_home",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Has the participant given permission for studystaff <b>to make home visits</b> for follow-up purposes?",
                    ),
                ),
                (
                    "may_sms",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="Has the participant given permission <b>to be contacted by SMS</b> by study staff for follow-up purposes during the study?",
                    ),
                ),
                (
                    "mail_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Mailing address ",
                    ),
                ),
                (
                    "physical_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Physical address with detailed description",
                    ),
                ),
                (
                    "subject_cell",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number",
                    ),
                ),
                (
                    "subject_cell_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number (alternate)",
                    ),
                ),
                (
                    "subject_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone",
                    ),
                ),
                (
                    "subject_phone_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone (alternate)",
                    ),
                ),
                (
                    "subject_work_place",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=250,
                        null=True,
                        verbose_name="Name and location of work place",
                    ),
                ),
                (
                    "subject_work_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Work contact number ",
                    ),
                ),
                (
                    "may_contact_indirectly",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="For example a partner, spouse, family member, neighbour ...",
                        max_length=25,
                        verbose_name="Has the participant given permission for study staff <b>to contact anyone else</b> for follow-up purposes during the study?",
                    ),
                ),
                (
                    "indirect_contact_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Full names of the contact person",
                    ),
                ),
                (
                    "indirect_contact_relation",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Relationship to participant",
                    ),
                ),
                (
                    "indirect_contact_physical_address",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=500,
                        null=True,
                        verbose_name="Full physical address ",
                    ),
                ),
                (
                    "indirect_contact_cell",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number",
                    ),
                ),
                (
                    "indirect_contact_cell_alt",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.cell_number],
                        verbose_name="Cell number (alternative)",
                    ),
                ),
                (
                    "indirect_contact_phone",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.telephone_number],
                        verbose_name="Telephone number",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sites.Site",
                    ),
                ),
            ],
            options={"abstract": False},
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from django.contrib import admin

from ..admin_site import edc_lab_admin
from ..models import Order
from .base_model_admin import BaseModelAdmin


@admin.register(Order, site=edc_lab_admin)
class OrderAdmin(BaseModelAdmin, admin.ModelAdmin):
    pass



# Generated by Django 5.2.6 on 2025-09-17 16:53

import django.db.models.deletion
import edc_model_fields.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_appointment", "0049_alter_appointment_revision_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appointment",
            name="appt_type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_appointment.appointmenttype",
                verbose_name="Appointment type",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="appt_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                verbose_name="If other appointment type, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="comment",
            field=models.CharField(
                blank=True, default="", max_length=250, verbose_name="Comment"
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="document_status_comments",
            field=models.TextField(
                blank=True,
                default="",
                help_text="for example, why some data is still pending",
                verbose_name="Any comments related to status of this document",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="appointmenttype",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="appointmenttype",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="appointmenttype",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_appointment.appointmenttype",
                verbose_name="Appointment type",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                verbose_name="If other appointment type, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="comment",
            field=models.CharField(
                blank=True, default="", max_length=250, verbose_name="Comment"
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="document_status_comments",
            field=models.TextField(
                blank=True,
                default="",
                help_text="for example, why some data is still pending",
                verbose_name="Any comments related to status of this document",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="infosources",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="infosources",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="infosources",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
    ]
